#ifndef DISPLAY_COMPONENT_H
#define DISPLAY_COMPONENT_H

#include <comdel/domain/instance.h>
#include <comdel/domain/wireinstance.h>

#include <QGraphicsItemGroup>
#include "comdel/domain/connectioninstance.h"

namespace display {

class Pin: public QGraphicsItemGroup
{
private:
    domain::Pin pin;
public:
    Pin(domain::Pin pin): pin(pin) {
        pin.getDisplayPin().render(this);
    }
};

class Component: public QGraphicsItemGroup
{
private:
    std::shared_ptr<domain::ComponentInstance> instance;
public:
    Component(const std::shared_ptr<domain::ComponentInstance>& instance): instance(instance) {
        setFlag(ItemSendsGeometryChanges, true);

        instance->component.getDisplay().render(this);
    }
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event) override;

};

class Bus: public QGraphicsItemGroup
{
    std::shared_ptr<domain::BusInstance> busInstance;
public:
    Bus(const std::shared_ptr<domain::BusInstance>& instance): busInstance(instance) {
        instance->bus.getDisplay()->render(this);
    }
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event) override;

};

class ComponentGroup: public QGraphicsItemGroup
{
private:
    std::shared_ptr<domain::ComponentInstance> componentInstance;

public:
    explicit ComponentGroup(const std::shared_ptr<domain::ComponentInstance>& instance): componentInstance(instance) {
        setFlag(ItemIsMovable, true);
        setFlag(ItemSendsGeometryChanges, true);

        setHandlesChildEvents(false);

        addToGroup(new display::Component(instance));

        for(auto &pin: instance->component.getPins()) {
            addToGroup(new display::Pin(pin));
        }

        setPos(instance->position.first, instance->position.second);
    }

protected:
    virtual QVariant itemChange(GraphicsItemChange change, const QVariant &value);

};

class BusGroup: public QGraphicsItemGroup
{
private:
    std::shared_ptr<domain::BusInstance> busInstance;

public:
    explicit BusGroup(const std::shared_ptr<domain::BusInstance>& instance): busInstance(instance) {
        setFlag(ItemIsMovable, true);
        setFlag(ItemSendsGeometryChanges, true);

        setHandlesChildEvents(false);

        addToGroup(new display::Bus(instance));

        setPos(instance->position.first, instance->position.second);
    }

protected:
    virtual QVariant itemChange(GraphicsItemChange change, const QVariant &value);
};

class BusConnection: public QGraphicsLineItem
{
private:
    domain::BusConnectionInstance* connection;
    ComponentGroup* component;
    BusGroup* bus;

public:
    BusConnection(domain::BusConnectionInstance* connection, ComponentGroup *component, BusGroup *bus): connection(connection), component(component), bus(bus) {
        updateConnection();

        setHandlesChildEvents(false);
    }

    void updateConnection() {
        auto busPosition = bus->boundingRect();
        auto pin = connection->instance->component.getPin(connection->connection.getComponent().pin).getDisplayPin();

        setLine(connection->instance->position.first + pin.getConnectionX(), connection->instance->position.second + pin.getConnectionY(),  connection->bus->position.first + busPosition.width()/2, connection->bus->position.second + busPosition.height()/2);
    }

    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event) override;
};


} // namespace display

#endif // DISPLAY_COMPONENT_H
